from celery import shared_task

from .signals import slack_event_received


@shared_task
def receive_slack_signal_task(sender, event_type, event_data, **data):
    slack_event_received.send(sender=sender, event_type=event_type, event_data=event_data, **data)




from django.urls import path

from .views import slack_oauthcallback, slack_login_callback, slack_interactivity, slack_command, connect_account, \
    slack_events

app_name = "slack_app"

urlpatterns = [
    path('install/', slack_oauthcallback, name='install'),
    path('login/', slack_login_callback, name='login'),
    path('interactivity/', slack_interactivity, name='interactivity'),
    path('events/', slack_events, name='events'),
    path('commands/<str:name>/', slack_command, name='command'),
    path('connect/<str:nonce>/', connect_account, name='connect_account'),
]


import datetime
import hashlib
import hmac
import time
from typing import Tuple

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.http import HttpRequest

from .models import SlackWorkspace, SlackWebHook, SlackUserMapping

slack_interactivity_callbacks = dict()
slack_commands = dict()


def create_workspace_from_oauth2_response(request_user, response) -> Tuple[SlackWorkspace, SlackWebHook]:
    workspace, created = SlackWorkspace.objects.update_or_create(
        id=response.get("team").get("id"),
        defaults={
            "name": response.get("team").get("name"),
            "bot_access_token": response.get("access_token"),
            "bot_user_id": response.get("bot_user_id"),
            "last_changed": datetime.datetime.now(),
            "response": response,
            "scope": response.get("scope"),
        }
    )

    workspace.update_slack_metadata()

    SlackUserMapping.objects.filter(slack_team_id=workspace.id).update(slack_workspace=workspace)
    workspace.owners.add(request_user)
    webhook_data = response.get("incoming_webhook")

    hook = SlackWebHook.objects.create(
        workspace=workspace,
        channel_id=webhook_data.get("channel_id"),
        channel_name=webhook_data.get("channel"),
        configuration_url=webhook_data.get("configuration_url"),
        url=webhook_data.get("url"),
    )

    return workspace, hook


def is_verified_slack_request(request: HttpRequest) -> bool:
    """
    Implements Slack signature verification according to
    https://api.slack.com/docs/verifying-requests-from-slack

    :param request: Django request
    :return: True if request is verified, False otherwise
    """
    try:
        slack_signing_secret = settings.SLACK_SIGNING_SECRET
    except AttributeError:
        raise ImproperlyConfigured(
            "SLACK_SIGNING_SECRET is missing in your settings.py" +
            "Please check the readme of django-slack-integration package"
        )

    slack_signature = request.META.get('HTTP_X_SLACK_SIGNATURE')
    slack_request_timestamp = request.META.get('HTTP_X_SLACK_REQUEST_TIMESTAMP')

    if not slack_signature or not slack_request_timestamp:
        return False

    if abs(time.time() - int(slack_request_timestamp)) > 60 * 5:
        # The request timestamp is more than five minutes from local time.
        # It could be a replay attack, so let's ignore it.
        return False

    sig_basestring = f"v0:{slack_request_timestamp}:{request.body.decode('utf-8')}".encode("utf-8")
    my_signature = 'v0=' + hmac.new(bytes(slack_signing_secret, 'utf-8'), sig_basestring, hashlib.sha256).hexdigest()

    return hmac.compare_digest(my_signature, slack_signature)



import slack
from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.auth.backends import BaseBackend
from django.db import transaction

from .models import SlackUserMapping, SlackWorkspace

User = get_user_model()


class SlackAuthenticationBackend(BaseBackend):

    @transaction.atomic
    def authenticate(self, request, code=None, redirect_uri=None, *args, **kwargs):
        if code is None:
            return None

        client = slack.WebClient(token="")

        response = client.oauth_access(
            client_id=settings.SLACK_CLIENT_ID,
            client_secret=settings.SLACK_CLIENT_SECRET,
            code=code,
            redirect_uri=redirect_uri
        )

        data = response.data

        if not data.get("ok"):
            return None

        user_id = data.get("user_id")

        try:
            slack_workspace = SlackWorkspace.objects.get(id=data.get("team_id"))
        except SlackWorkspace.DoesNotExist:
            slack_workspace = None

        slack_user, created = SlackUserMapping.objects.update_or_create(
            slack_user_id=user_id,
            defaults={
                "access_token": data.get("access_token"),
                "slack_team_id": data.get("team_id"),
                "slack_workspace": slack_workspace,  # shortcut for each access, on_delete=models.SET_NULL
            }
        )

        if slack_user.user is None:
            username = data.get("user").get("name")
            # TODO get an unique username in a more sophisticated way
            n = 1
            while User.objects.filter(username=username).exists():
                username = f"{data.get('user').get('name')}-{n}"
                n += 1

            user = User.objects.create(username=username)

            slack_user.user = user
            slack_user.save()

        return slack_user.user

    def get_user(self, user_id):
        try:
            return User.objects.get(pk=user_id)
        except User.DoesNotExist:
            return None



import django.dispatch

"""
Slack want us to implement a queue to react to events.
https://api.slack.com/events-api#responding_to_events

That's why, we've chosen to implement this using Django's signals.
"""
slack_event_received = django.dispatch.Signal(providing_args=["event_type", "event_data"])
refresh_home = django.dispatch.Signal(providing_args=["slack_user_mapping", "slack_workspace"])



# Generated by Django 3.0.3 on 2020-02-07 21:28

from django.conf import settings
import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='SlackWorkspace',
            fields=[
                ('id', models.CharField(max_length=255, primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=255)),
                ('scope', models.CharField(max_length=1024)),
                ('bot_access_token', models.CharField(max_length=255)),
                ('bot_user_id', models.CharField(max_length=255)),
                ('last_changed', models.DateTimeField(auto_now_add=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('response', django.contrib.postgres.fields.jsonb.JSONField()),
                ('domain', models.CharField(max_length=255, null=True)),
                ('image_34', models.URLField(null=True)),
                ('image_44', models.URLField(null=True)),
                ('image_68', models.URLField(null=True)),
                ('image_88', models.URLField(null=True)),
                ('image_102', models.URLField(null=True)),
                ('image_132', models.URLField(null=True)),
                ('image_230', models.URLField(null=True)),
                ('image_original', models.URLField(null=True)),
                ('image_default', models.BooleanField(null=True)),
                ('enterprise_id', models.CharField(max_length=128, null=True)),
                ('enterprise_name', models.CharField(max_length=128, null=True)),
                ('owners', models.ManyToManyField(related_name='owned_slack_workspaces', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='SlackWebHook',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('channel_name', models.CharField(max_length=255)),
                ('channel_id', models.CharField(max_length=255)),
                ('configuration_url', models.URLField(max_length=255)),
                ('url', models.URLField(max_length=255)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('workspace', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='webhooks', to='slack_app.SlackWorkspace')),
            ],
        ),
        migrations.CreateModel(
            name='SlackUserMapping',
            fields=[
                ('slack_user_id', models.CharField(db_index=True, max_length=255, primary_key=True, serialize=False)),
                ('slack_team_id', models.CharField(max_length=255)),
                ('access_token', models.CharField(max_length=255)),
                ('nonce', models.UUIDField(default=uuid.uuid4)),
                ('slack_email', models.EmailField(max_length=254, null=True)),
                ('image_24', models.URLField(null=True)),
                ('image_32', models.URLField(null=True)),
                ('image_48', models.URLField(null=True)),
                ('image_72', models.URLField(null=True)),
                ('image_192', models.URLField(null=True)),
                ('image_512', models.URLField(null=True)),
                ('image_1024', models.URLField(null=True)),
                ('workspace_name', models.CharField(max_length=255)),
                ('workspace_domain', models.CharField(max_length=255)),
                ('workspace_image_34', models.URLField(null=True)),
                ('workspace_image_44', models.URLField(null=True)),
                ('workspace_image_68', models.URLField(null=True)),
                ('workspace_image_88', models.URLField(null=True)),
                ('workspace_image_102', models.URLField(null=True)),
                ('workspace_image_132', models.URLField(null=True)),
                ('workspace_image_230', models.URLField(null=True)),
                ('workspace_image_original', models.URLField(null=True)),
                ('slack_workspace', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='slack_users', to='slack_app.SlackWorkspace')),
                ('user', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='slack_accounts', to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



from django.conf import settings

SLACK_LOGIN_OAUTH_REDIRECT_URL = getattr(settings, 'SLACK_LOGIN_OAUTH_REDIRECT_URL', "/")
SLACK_INSTALL_OAUTH_REDIRECT_URL = getattr(settings, 'SLACK_INSTALL_OAUTH_REDIRECT_URL', "/")



from functools import wraps

import slack
from django.core.exceptions import ImproperlyConfigured
from django.http import HttpResponseBadRequest

from .signals import slack_event_received, refresh_home
from .models import SlackUserMapping, SlackWorkspace

from .helpers import is_verified_slack_request, slack_interactivity_callbacks, slack_commands


def slack_command(name, require_linked_account=True):
    def _decorator(handler_func):
        if slack_commands.get(name, None):
            raise ImproperlyConfigured(f"You are trying to connect one slash command to multiple functions.")
        slack_commands[name] = (handler_func, require_linked_account)
        return handler_func

    return _decorator


def slack_interactivity(interactivity_type, require_linked_account=True):
    def _decorator(handler_func):
        if slack_interactivity_callbacks.get(interactivity_type, None):
            raise ImproperlyConfigured(f"You are trying to connect '{interactivity_type}' in multiple functions.")
        slack_interactivity_callbacks[interactivity_type] = (handler_func, require_linked_account)
        return handler_func

    return _decorator


def slack_verify_request(view_func):
    def wrapped_view(request, *args, **kwargs):
        if is_verified_slack_request(request):
            return view_func(request, *args, **kwargs)
        else:
            return HttpResponseBadRequest("Slack signature verification failed")

    # we verify that the request is coming from Slack, csrf is irrelevant, but most importantly not provided at all
    view_func.csrf_exempt = True

    return wraps(view_func)(wrapped_view)


def on_slack_signal(*event_types, inject_slack_models=False):
    def decorator_receiver(receiver_func):
        @wraps(receiver_func)
        def signal_receiver(sender, event_type, event_data, signal, **kwargs):
            if event_type in event_types:
                try:
                    slack_user_mapping = SlackUserMapping.objects.get(pk=event_data.get('user'))
                except SlackUserMapping.DoesNotExist:
                    slack_user_mapping = None

                try:
                    slack_workspace = SlackWorkspace.objects.get(pk=kwargs.get('team_id'))
                except SlackWorkspace.DoesNotExist:
                    slack_workspace = None

                slack_models = {
                    "slack_user_mapping": slack_user_mapping,
                    "slack_workspace": slack_workspace,
                } if inject_slack_models else {}
                return receiver_func(sender, event_data=event_data, event_type=event_type, **slack_models, **kwargs)

        slack_event_received.connect(signal_receiver, weak=False)

    return decorator_receiver


def slack_app_home(receiver_func):
    @wraps(receiver_func)
    def signal_receiver(
        sender,
        event_type=None,
        event_data=None,

        slack_user_mapping=None,
        slack_workspace=None,
        **kwargs
    ):
        if event_type == 'app_home_opened':
            slack_user_mapping = SlackUserMapping.objects.get(pk=event_data.get('user'))
            slack_workspace = SlackWorkspace.objects.get(pk=kwargs.get('team_id'))

        blocks, title = receiver_func(
            sender,
            event_data=event_data,
            event_type=event_type,
            slack_user_mapping=slack_user_mapping,
            slack_workspace=slack_workspace,
            **kwargs,
        )

        client = slack.WebClient(token=slack_workspace.bot_access_token)
        client.views_publish(user_id=slack_user_mapping.slack_user_id, view={
            "type": 'home',
            "title": title,
            "blocks": blocks
        })

    slack_event_received.connect(signal_receiver, weak=False)
    refresh_home.connect(signal_receiver, weak=False)

    return signal_receiver



import json
from typing import Tuple

import slack
from django.conf import settings
from django.contrib.auth import authenticate, login
from django.contrib.auth.decorators import login_required
from django.db import transaction
from django.http import HttpResponseRedirect, HttpResponse, JsonResponse
from django.shortcuts import render, get_object_or_404
from django.urls import reverse
from django.views.decorators.http import require_http_methods

from .tasks import receive_slack_signal_task
from .exceptions import SlackAppNotInstalledProperlyException, SlackAccountNotLinkedException, SlackCommandDoesNotExist, \
    SlackInteractivityTypeDoesNotExist
from .models import SlackWorkspace, SlackUserMapping
from .decorators import slack_verify_request
from .settings import SLACK_LOGIN_OAUTH_REDIRECT_URL, SLACK_INSTALL_OAUTH_REDIRECT_URL
from .helpers import create_workspace_from_oauth2_response, slack_interactivity_callbacks, slack_commands


@login_required  # we make sure user is logged-in in order to link Slack
@require_http_methods(["GET"])  # GET is what Slack invokes, let's forbid others just to be sure
@transaction.atomic  # whatever happens in a db, let's behave atomically
@slack_verify_request
def slack_oauthcallback(request):
    if "error" in request.GET:
        return HttpResponseRedirect(SLACK_INSTALL_OAUTH_REDIRECT_URL)  # TODO - pass GET params

    code = request.GET.get("code")

    client = slack.WebClient(token="")

    response = client.oauth_v2_access(
        client_id=settings.SLACK_CLIENT_ID,
        client_secret=settings.SLACK_CLIENT_SECRET,
        code=code
    )

    workspace, hook = create_workspace_from_oauth2_response(request.user, response.data)

    return HttpResponseRedirect(SLACK_INSTALL_OAUTH_REDIRECT_URL)  # TODO - pass workspace & hook id parameters


@require_http_methods(["GET"])  # GET is what SEND invokes, let's forbid others just to be sure
@slack_verify_request
def slack_login_callback(request):
    if "error" in request.GET:
        return render(SLACK_LOGIN_OAUTH_REDIRECT_URL)  # TODO - pass GET params

    code = request.GET["code"]

    user = authenticate(
        request,
        code=code,
        redirect_uri=request.build_absolute_uri(reverse("slack_app:login"))
    )

    if user is not None:
        login(request, user)

    return HttpResponseRedirect(SLACK_LOGIN_OAUTH_REDIRECT_URL)


@slack_verify_request
@require_http_methods(["POST"])
def slack_interactivity(request):
    payload = json.loads(request.POST.get('payload'))
    payload_type = payload.get("type")
    callback = slack_interactivity_callbacks.get(payload_type, None)

    if callback is None:
        raise SlackInteractivityTypeDoesNotExist(
            f"Interactivity type '{payload_type}' is not linked using @slack_interactivity decorator")

    fn, required_linked_account = callback
    if fn:
        if required_linked_account:
            try:
                mapping, workspace = get_slack_user_and_workspace(
                    payload.get('team').get('id'),
                    payload.get('user').get('id'),
                )
            except SlackAppNotInstalledProperlyException:
                return JsonResponse({
                    "text": "Slack application is not linked to your workspace. " +
                            "Please ask your administrators to finish the installation."
                })
            except SlackAccountNotLinkedException as err:
                url = reverse('slack_app:connect_account', kwargs={'nonce': err.slack_user_mapping.nonce})
                return JsonResponse({
                    "text": "Hi, it seems like you haven't linked your Slack account to your Scrumie account. " +
                            f"You can do so <{request.build_absolute_uri(url)}|here>"
                })
        else:
            mapping = None
            workspace = None
        return fn(request, payload, mapping, workspace)

    return HttpResponse(status=400)


def get_slack_user_and_workspace(team_id, user_id) -> Tuple[SlackUserMapping, SlackWorkspace]:
    try:
        workspace = SlackWorkspace.objects.get(id=team_id)
    except SlackWorkspace.DoesNotExist:
        raise SlackAppNotInstalledProperlyException("Application is not installed properly")

    mapping, created = SlackUserMapping.objects.get_or_create(slack_user_id=user_id, slack_workspace=workspace)

    if created or mapping.user is None:
        raise SlackAccountNotLinkedException(mapping)

    return mapping, workspace


@slack_verify_request
@require_http_methods(["POST"])
def slack_command(request, name: str):
    payload = request.POST
    callback = slack_commands.get(name, None)

    if callback is None:
        raise SlackCommandDoesNotExist(f"Command '{name}' is not linked using @slack_command decorator")

    fn, required_linked_account = callback
    if fn:
        if required_linked_account:
            try:
                mapping, workspace = get_slack_user_and_workspace(
                    request.POST.get('team_id'),
                    request.POST.get('user_id')
                )
            except SlackAppNotInstalledProperlyException:
                return JsonResponse({
                    "text": "Slack application is not linked to your workspace. " +
                            "Please ask your administrators to finish the installation."
                })
            except SlackAccountNotLinkedException as err:
                url = reverse('slack_app:connect_account', kwargs={'nonce': err.slack_user_mapping.nonce})
                return JsonResponse({
                    "text": "Hi, it seems like you haven't linked your Slack account to your Scrumie account. " +
                            f"You can do so <{request.build_absolute_uri(url)}|here>"
                })
        else:
            workspace = None
            mapping = None

        return fn(request, payload, mapping, workspace)

    return HttpResponse(status=400)


@login_required
def connect_account(request, nonce: str):
    mapping = get_object_or_404(SlackUserMapping, nonce=nonce)
    mapping.user = request.user
    mapping.save()

    # TODO - allow to specify a function that accepts request, mapping & user and return whatever it wants
    # (Redirect/Render, etc)
    return render(request, "verified_nonce.html")


@slack_verify_request
def slack_events(request):
    data = json.loads(request.body)

    if data.get("type") == "url_verification":
        return JsonResponse({
            "challenge": data.get("challenge"),
        })

    if data.get("type") == "event_callback":
        event_data = data.pop('event')
        event_type = event_data.pop('type')

        receive_slack_signal_task.delay(sender=request.get_host(), event_type=event_type, event_data=event_data, **data)
        return JsonResponse({})

    if data.get("type") == 'app_rate_limited':
        return JsonResponse({})

    return NotImplementedError(f"Unknown type {type}")


from django.apps import AppConfig
from django.utils.module_loading import autodiscover_modules


class SlackAppConfig(AppConfig):
    name = 'slack_app'
    verbose_name = 'Slack Integration'

    def ready(self):
        super().ready()

        autodiscover_modules('slack')



import uuid
import slack

from django.conf import settings
from django.contrib.auth import get_user_model, user_logged_in
from django.contrib.postgres.fields import JSONField
from django.db import models
from django.db.models.signals import pre_delete
from django.dispatch import receiver
from slack.errors import SlackApiError

User = get_user_model()


class SlackWorkspace(models.Model):
    id = models.CharField(primary_key=True, max_length=255)
    name = models.CharField(max_length=255)
    scope = models.CharField(max_length=1024)
    bot_access_token = models.CharField(max_length=255)
    bot_user_id = models.CharField(max_length=255)
    last_changed = models.DateTimeField(auto_now_add=True)
    created = models.DateTimeField(auto_now_add=True)
    response = JSONField()

    owners = models.ManyToManyField(User, related_name='owned_slack_workspaces')

    domain = models.CharField(max_length=255, null=True)
    image_34 = models.URLField(null=True)
    image_44 = models.URLField(null=True)
    image_68 = models.URLField(null=True)
    image_88 = models.URLField(null=True)
    image_102 = models.URLField(null=True)
    image_132 = models.URLField(null=True)
    image_230 = models.URLField(null=True)
    image_original = models.URLField(null=True)
    image_default = models.BooleanField(null=True)

    enterprise_id = models.CharField(max_length=128, null=True)
    enterprise_name = models.CharField(max_length=128, null=True)

    def get_bot_scopes(self):
        return self.scope.split(',') if self.scope else []

    def update_slack_metadata(self):
        """
        Called to re-fetch data about this entity from Slack API
        :return:
        """
        client = slack.WebClient(token=self.bot_access_token)
        res = client.team_info()

        if res.get('ok'):
            team_data = res.get("team")

            self.name = team_data.get("name")
            self.domain = team_data.get("domain")
            self.email_domain = team_data.get("email_domain")
            self.enterprise_id = team_data.get("enterprise_id")
            self.enterprise_name = team_data.get("enterprise_name")
            self.image_34 = team_data.get("icon").get("image_34")
            self.image_44 = team_data.get("icon").get("image_44")
            self.image_68 = team_data.get("icon").get("image_68")
            self.image_88 = team_data.get("icon").get("image_88")
            self.image_102 = team_data.get("icon").get("image_102")
            self.image_132 = team_data.get("icon").get("image_132")
            self.image_230 = team_data.get("icon").get("image_230")
            self.image_original = team_data.get("icon").get("image_original")
            self.image_default = team_data.get("icon").get("image_default", False)
            self.save()

    def __str__(self):
        return self.name


class SlackWebHook(models.Model):
    workspace = models.ForeignKey(SlackWorkspace, on_delete=models.CASCADE, related_name='webhooks')
    channel_name = models.CharField(max_length=255)
    channel_id = models.CharField(max_length=255)
    configuration_url = models.URLField(max_length=255)
    url = models.URLField(max_length=255)
    created = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return f"{self.channel_name}@{self.workspace.name}"


class SlackUserMapping(models.Model):
    slack_user_id = models.CharField(max_length=255, db_index=True, primary_key=True)
    slack_team_id = models.CharField(max_length=255)

    # user can be null till it's unassigned
    user = models.ForeignKey(User, on_delete=models.CASCADE, null=True, related_name='slack_accounts')

    # If slack app is uninstalled, all of the tokens are uninstalled as well
    slack_workspace = models.ForeignKey(SlackWorkspace, on_delete=models.CASCADE, related_name='slack_users', null=True)

    access_token = models.CharField(max_length=255)
    nonce = models.UUIDField(max_length=36, default=uuid.uuid4)

    slack_email = models.EmailField(null=True)
    image_24 = models.URLField(null=True)
    image_32 = models.URLField(null=True)
    image_48 = models.URLField(null=True)
    image_72 = models.URLField(null=True)
    image_192 = models.URLField(null=True)
    image_512 = models.URLField(null=True)
    image_1024 = models.URLField(null=True)

    workspace_name = models.CharField(max_length=255)
    workspace_domain = models.CharField(max_length=255)
    workspace_image_34 = models.URLField(null=True)
    workspace_image_44 = models.URLField(null=True)
    workspace_image_68 = models.URLField(null=True)
    workspace_image_88 = models.URLField(null=True)
    workspace_image_102 = models.URLField(null=True)
    workspace_image_132 = models.URLField(null=True)
    workspace_image_230 = models.URLField(null=True)
    workspace_image_original = models.URLField(null=True)

    def update_slack_metadata(self):
        """
        Called to re-fetch data about this entity from Slack API
        :return:
        """
        client = slack.WebClient(token=self.access_token)
        res = client.users_identity()

        if res.get('ok'):
            user_data = res.get("user")
            team_data = res.get("team")

            self.name = user_data.get("name")
            self.slack_email = user_data.get("email")

            self.image_24 = user_data.get("image_24")
            self.image_32 = user_data.get("image_32")
            self.image_48 = user_data.get("image_48")
            self.image_72 = user_data.get("image_72")
            self.image_192 = user_data.get("image_192")
            self.image_512 = user_data.get("image_512")
            self.image_1024 = user_data.get("image_1024")

            self.workspace_name = team_data.get("name")
            self.workspace_domain = team_data.get("domain")
            self.workspace_image_34 = team_data.get("image_34")
            self.workspace_image_44 = team_data.get("image_44")
            self.workspace_image_68 = team_data.get("image_68")
            self.workspace_image_88 = team_data.get("image_88")
            self.workspace_image_102 = team_data.get("image_102")
            self.workspace_image_132 = team_data.get("image_132")
            self.workspace_image_230 = team_data.get("image_230")
            self.workspace_image_original = team_data.get("image_original")

            self.save()

    def __str__(self):
        username = self.user.username if self.user else 'Unassigned Account'
        return username


@receiver(pre_delete, sender=SlackWorkspace)
def post_delete_slack_workspace(instance, *args, **kwargs):
    """
    Once the application is removed from Django, let's remove it from Slack's workspace
    """
    client = slack.WebClient(token=instance.bot_access_token)
    try:
        client.api_call('apps.uninstall', params={
            "client_id": settings.SLACK_CLIENT_ID,
            "client_secret": settings.SLACK_CLIENT_SECRET,
        })
    except SlackApiError:
        # for now fail silently
        pass


@receiver(user_logged_in)
def refetch_user_slack_metadata(sender, user, request, **kwargs):
    """
    We will re-fetch user's data after a successful login.
    """
    for slack_user in user.slack_accounts.all():
        try:
            slack_user.update_slack_metadata()
        except:
            # for now fail silently, since this is login related, we should catch all errors just to be sure
            pass



from django import forms


class SlackCommandForm(forms.Form):
    token = forms.CharField()
    command = forms.CharField()

    team_id = forms.CharField(required=False)
    team_domain = forms.CharField(required=False)

    channel_id = forms.CharField(required=False)
    channel_name = forms.CharField(required=False)

    user_id = forms.CharField()
    user_name = forms.CharField()

    response_url = forms.URLField()
    trigger_id = forms.CharField()

    enterprise_id = forms.CharField(required=False)
    enterprise_name = forms.CharField(required=False)



class SlackAppNotInstalledProperlyException(Exception):
    """
    Thrown if no entry for a Slack app exists in a Django DB.
    """
    pass


class SlackAccountNotLinkedException(Exception):
    """
    Thrown if user is not linked with a User model yet.
    """

    def __init__(self, slack_user_mapping):
        super().__init__()
        self.slack_user_mapping = slack_user_mapping


class SlackReceiverDoesNotExist(Exception):
    pass


class SlackInteractivityTypeDoesNotExist(SlackReceiverDoesNotExist):
    pass


class SlackCommandDoesNotExist(SlackReceiverDoesNotExist):
    pass
